@@ -5,7 +5,7 @@
  */
 
 #include "config.h"
-#include "adw-action-row.h"
+#include "adw-action-row-private.h"
 
 #include "adw-marshalers.h"
 #include "adw-widget-utils-private.h"
@@ -47,6 +47,20 @@
  *
  * It contains subnodes `label.title` and `label.subtitle` representing
  * respectively the title label and subtitle label.
+ *
+ * ## Style classes
+ *
+ * `AdwActionRow` can use the [`.property`](style-classes.html#property-rows)
+ * style class to emphasize the row subtitle instead of the row title, which is
+ * useful for displaying read-only properties.
+ *
+ * <picture>
+ *   <source srcset="property-row-dark.png" media="(prefers-color-scheme: dark)">
+ *   <img src="property-row.png" alt="property-row">
+ * </picture>
+ *
+ * When used together with the `.monospace` style class, only the subtitle
+ * becomes monospace, not the title or any extra widgets.
  */
 
 typedef struct
@@ -104,6 +118,47 @@ string_is_not_empty (AdwActionRow *self,
   return string && string[0];
 }
 
+static void
+pressed_cb (AdwActionRow *self,
+            int           n_press,
+            double        x,
+            double        y,
+            GtkGesture   *gesture)
+{
+  AdwActionRowPrivate *priv = adw_action_row_get_instance_private (self);
+  GtkWidget *picked;
+  GtkEditable *delegate;
+
+  picked = gtk_widget_pick (GTK_WIDGET (self), x, y, GTK_PICK_DEFAULT);
+
+  if (picked != GTK_WIDGET (self) &&
+      picked != priv->header &&
+      picked != GTK_WIDGET (priv->prefixes) &&
+      picked != GTK_WIDGET (priv->suffixes)) {
+    gtk_gesture_set_state (gesture, GTK_EVENT_SEQUENCE_DENIED);
+
+    return;
+  }
+
+  if (!GTK_IS_EDITABLE (self)) {
+    gtk_gesture_set_state (gesture, GTK_EVENT_SEQUENCE_DENIED);
+    return;
+  }
+
+  delegate = gtk_editable_get_delegate (GTK_EDITABLE (self));
+
+  while (delegate && !GTK_IS_TEXT (delegate))
+    delegate = gtk_editable_get_delegate (delegate);
+
+  if (!delegate || !GTK_IS_TEXT (delegate)) {
+    gtk_gesture_set_state (gesture, GTK_EVENT_SEQUENCE_DENIED);
+    return;
+  }
+
+  gtk_text_grab_focus_without_selecting (GTK_TEXT (delegate));
+  gtk_gesture_set_state (gesture, GTK_EVENT_SEQUENCE_CLAIMED);
+}
+
 static void
 row_activated_cb (AdwActionRow  *self,
                   GtkListBoxRow *row)
@@ -235,7 +290,7 @@ adw_action_row_class_init (AdwActionRowClass *klass)
   klass->activate = adw_action_row_activate_real;
 
   /**
-   * AdwActionRow:subtitle: (attributes org.gtk.Property.get=adw_action_row_get_subtitle org.gtk.Property.set=adw_action_row_set_subtitle)
+   * AdwActionRow:subtitle:
    *
    * The subtitle for this row.
    *
@@ -248,7 +303,7 @@ adw_action_row_class_init (AdwActionRowClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwActionRow:icon-name: (attributes org.gtk.Property.get=adw_action_row_get_icon_name org.gtk.Property.set=adw_action_row_set_icon_name)
+   * AdwActionRow:icon-name:
    *
    * The icon name for this row.
    *
@@ -257,15 +312,15 @@ adw_action_row_class_init (AdwActionRowClass *klass)
   props[PROP_ICON_NAME] =
     g_param_spec_string ("icon-name", NULL, NULL,
                          "",
-                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_DEPRECATED);
 
   /**
-   * AdwActionRow:activatable-widget: (attributes org.gtk.Property.get=adw_action_row_get_activatable_widget org.gtk.Property.set=adw_action_row_set_activatable_widget)
+   * AdwActionRow:activatable-widget:
    *
    * The widget to activate when the row is activated.
    *
    * The row can be activated either by clicking on it, calling
-   * [method@ActionRow.activate], or via mnemonics in the title or the subtitle.
+   * [method@ActionRow.activate], or via mnemonics in the title.
    * See the [property@PreferencesRow:use-underline] property to enable
    * mnemonics.
    *
@@ -278,7 +333,7 @@ adw_action_row_class_init (AdwActionRowClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwActionRow:title-lines: (attributes org.gtk.Property.get=adw_action_row_get_title_lines org.gtk.Property.set=adw_action_row_set_title_lines)
+   * AdwActionRow:title-lines:
    *
    * The number of lines at the end of which the title label will be ellipsized.
    *
@@ -291,7 +346,7 @@ adw_action_row_class_init (AdwActionRowClass *klass)
                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwActionRow:subtitle-lines: (attributes org.gtk.Property.get=adw_action_row_get_subtitle_lines org.gtk.Property.set=adw_action_row_set_subtitle_lines)
+   * AdwActionRow:subtitle-lines:
    *
    * The number of lines at the end of which the subtitle label will be
    * ellipsized.
@@ -305,7 +360,7 @@ adw_action_row_class_init (AdwActionRowClass *klass)
                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwActionRow:subtitle-selectable: (attributes org.gtk.Property.get=adw_action_row_get_subtitle_selectable org.gtk.Property.set=adw_action_row_set_subtitle_selectable)
+   * AdwActionRow:subtitle-selectable:
    *
    * Whether the user can copy the subtitle from the label.
    *
@@ -348,6 +403,7 @@ adw_action_row_class_init (AdwActionRowClass *klass)
   gtk_widget_class_bind_template_child_private (widget_class, AdwActionRow, title);
   gtk_widget_class_bind_template_child_private (widget_class, AdwActionRow, title_box);
   gtk_widget_class_bind_template_callback (widget_class, string_is_not_empty);
+  gtk_widget_class_bind_template_callback (widget_class, pressed_cb);
 }
 
 static void
@@ -414,6 +470,7 @@ adw_action_row_add_prefix (AdwActionRow *self,
 
   g_return_if_fail (ADW_IS_ACTION_ROW (self));
   g_return_if_fail (GTK_IS_WIDGET (widget));
+  g_return_if_fail (gtk_widget_get_parent (widget) == NULL);
 
   priv = adw_action_row_get_instance_private (self);
 
@@ -436,6 +493,7 @@ adw_action_row_add_suffix (AdwActionRow *self,
 
   g_return_if_fail (ADW_IS_ACTION_ROW (self));
   g_return_if_fail (GTK_IS_WIDGET (widget));
+  g_return_if_fail (gtk_widget_get_parent (widget) == NULL);
 
   priv = adw_action_row_get_instance_private (self);
 
@@ -474,7 +532,7 @@ adw_action_row_remove (AdwActionRow *self,
 }
 
 /**
- * adw_action_row_get_subtitle: (attributes org.gtk.Method.get_property=subtitle)
+ * adw_action_row_get_subtitle:
  * @self: an action row
  *
  * Gets the subtitle for @self.
@@ -490,11 +548,11 @@ adw_action_row_get_subtitle (AdwActionRow *self)
 
   priv = adw_action_row_get_instance_private (self);
 
-  return gtk_label_get_text (priv->subtitle);
+  return gtk_label_get_label (priv->subtitle);
 }
 
 /**
- * adw_action_row_set_subtitle: (attributes org.gtk.Method.set_property=subtitle)
+ * adw_action_row_set_subtitle:
  * @self: an action row
  * @subtitle: the subtitle
  *
@@ -522,7 +580,7 @@ adw_action_row_set_subtitle (AdwActionRow *self,
 }
 
 /**
- * adw_action_row_get_icon_name: (attributes org.gtk.Method.get_property=icon-name)
+ * adw_action_row_get_icon_name:
  * @self: an action row
  *
  * Gets the icon name for @self.
@@ -544,7 +602,7 @@ adw_action_row_get_icon_name (AdwActionRow *self)
 }
 
 /**
- * adw_action_row_set_icon_name: (attributes org.gtk.Method.set_property=icon-name)
+ * adw_action_row_set_icon_name:
  * @self: an action row
  * @icon_name: (nullable): the icon name
  *
@@ -573,7 +631,7 @@ adw_action_row_set_icon_name (AdwActionRow *self,
 }
 
 /**
- * adw_action_row_get_activatable_widget: (attributes org.gtk.Method.get_property=activatable-widget)
+ * adw_action_row_get_activatable_widget:
  * @self: an action row
  *
  * Gets the widget activated when @self is activated.
@@ -606,14 +664,14 @@ activatable_widget_weak_notify (gpointer  data,
 }
 
 /**
- * adw_action_row_set_activatable_widget: (attributes org.gtk.Method.set_property=activatable-widget)
+ * adw_action_row_set_activatable_widget:
  * @self: an action row
  * @widget: (nullable): the target widget
  *
  * Sets the widget to activate when @self is activated.
  *
  * The row can be activated either by clicking on it, calling
- * [method@ActionRow.activate], or via mnemonics in the title or the subtitle.
+ * [method@ActionRow.activate], or via mnemonics in the title.
  * See the [property@PreferencesRow:use-underline] property to enable mnemonics.
  *
  * The target widget will be activated by emitting the
@@ -668,7 +726,7 @@ adw_action_row_set_activatable_widget (AdwActionRow *self,
 }
 
 /**
- * adw_action_row_get_title_lines: (attributes org.gtk.Method.get_property=title-lines)
+ * adw_action_row_get_title_lines:
  * @self: an action row
  *
  * Gets the number of lines at the end of which the title label will be
@@ -690,7 +748,7 @@ adw_action_row_get_title_lines (AdwActionRow *self)
 }
 
 /**
- * adw_action_row_set_title_lines: (attributes org.gtk.Method.set_property=title-lines)
+ * adw_action_row_set_title_lines:
  * @self: an action row
  * @title_lines: the number of lines at the end of which the title label will be ellipsized
  *
@@ -722,7 +780,7 @@ adw_action_row_set_title_lines (AdwActionRow *self,
 }
 
 /**
- * adw_action_row_get_subtitle_lines: (attributes org.gtk.Method.get_property=subtitle-lines)
+ * adw_action_row_get_subtitle_lines:
  * @self: an action row
  *
  * Gets the number of lines at the end of which the subtitle label will be
@@ -744,7 +802,7 @@ adw_action_row_get_subtitle_lines (AdwActionRow *self)
 }
 
 /**
- * adw_action_row_set_subtitle_lines: (attributes org.gtk.Method.set_property=subtitle-lines)
+ * adw_action_row_set_subtitle_lines:
  * @self: an action row
  * @subtitle_lines: the number of lines at the end of which the subtitle label will be ellipsized
  *
@@ -776,8 +834,8 @@ adw_action_row_set_subtitle_lines (AdwActionRow *self,
 }
 
 /**
- * adw_action_row_get_subtitle_selectable: (attributes org.gtk.Method.get_property=subtitle-selectable)
- * @self: a `AdwActionRow`
+ * adw_action_row_get_subtitle_selectable:
+ * @self: an action row
  *
  * Gets whether the user can copy the subtitle from the label
  *
@@ -796,8 +854,8 @@ adw_action_row_get_subtitle_selectable (AdwActionRow *self)
 }
 
 /**
- * adw_action_row_set_subtitle_selectable: (attributes org.gtk.Method.set_property=subtitle-selectable)
- * @self: a `AdwActionRow`
+ * adw_action_row_set_subtitle_selectable:
+ * @self: an action row
  * @subtitle_selectable: `TRUE` if the user can copy the subtitle from the label
  *
  * Sets whether the user can copy the subtitle from the label
@@ -837,3 +895,24 @@ adw_action_row_activate (AdwActionRow *self)
 
   ADW_ACTION_ROW_GET_CLASS (self)->activate (self);
 }
+
+void
+adw_action_row_set_expand_suffixes (AdwActionRow *self,
+                                    gboolean      expand)
+{
+  AdwActionRowPrivate *priv = adw_action_row_get_instance_private (self);
+
+  g_return_if_fail (ADW_IS_ACTION_ROW (self));
+
+  expand = !!expand;
+
+  if (expand) {
+    gtk_widget_set_hexpand (GTK_WIDGET (priv->title_box), FALSE);
+    gtk_label_set_natural_wrap_mode (priv->title, GTK_NATURAL_WRAP_NONE);
+    gtk_label_set_natural_wrap_mode (priv->subtitle, GTK_NATURAL_WRAP_NONE);
+  } else {
+    gtk_widget_set_hexpand (GTK_WIDGET (priv->title_box), TRUE);
+    gtk_label_set_natural_wrap_mode (priv->title, GTK_NATURAL_WRAP_INHERIT);
+    gtk_label_set_natural_wrap_mode (priv->subtitle, GTK_NATURAL_WRAP_INHERIT);
+  }
+}
