@@ -46,6 +46,8 @@ typedef struct
   char *name;
 
   gboolean use_underline;
+
+  GPtrArray *groups;
 } AdwPreferencesPagePrivate;
 
 static void adw_preferences_page_buildable_init (GtkBuildableIface *iface);
@@ -171,6 +173,7 @@ adw_preferences_page_finalize (GObject *object)
   g_clear_pointer (&priv->icon_name, g_free);
   g_clear_pointer (&priv->title, g_free);
   g_clear_pointer (&priv->name, g_free);
+  g_clear_pointer (&priv->groups, g_ptr_array_unref);
 
   G_OBJECT_CLASS (adw_preferences_page_parent_class)->finalize (object);
 }
@@ -287,6 +290,7 @@ adw_preferences_page_init (AdwPreferencesPage *self)
   gtk_orientable_set_orientation (GTK_ORIENTABLE (layout), GTK_ORIENTATION_VERTICAL);
 
   priv->title = g_strdup ("");
+  priv->groups = g_ptr_array_new ();
 
   gtk_widget_init_template (GTK_WIDGET (self));
 }
@@ -300,10 +304,15 @@ adw_preferences_page_buildable_add_child (GtkBuildable *buildable,
   AdwPreferencesPage *self = ADW_PREFERENCES_PAGE (buildable);
   AdwPreferencesPagePrivate *priv = adw_preferences_page_get_instance_private (self);
 
-  if (priv->box && ADW_IS_PREFERENCES_GROUP (child))
+  if (priv->box && ADW_IS_PREFERENCES_GROUP (child)) {
     adw_preferences_page_add (self, ADW_PREFERENCES_GROUP (child));
-  else
+  } else if (priv->scrolled_window && GTK_IS_WIDGET (child)) {
+    g_critical ("Trying to add %s as a child to an AdwPreferencePage, "
+                "but only AdwPreferencesPage is allowed",
+                G_OBJECT_TYPE_NAME (child));
+  } else {
     parent_buildable_iface->add_child (buildable, builder, child, type);
+  }
 }
 
 static void
@@ -344,6 +353,7 @@ adw_preferences_page_add (AdwPreferencesPage  *self,
 
   priv = adw_preferences_page_get_instance_private (self);
 
+  g_ptr_array_add (priv->groups, group);
   gtk_box_append (priv->box, GTK_WIDGET (group));
 }
 
@@ -365,10 +375,84 @@ adw_preferences_page_remove (AdwPreferencesPage  *self,
 
   priv = adw_preferences_page_get_instance_private (self);
 
-  if (gtk_widget_get_parent (GTK_WIDGET (group)) == GTK_WIDGET (priv->box))
-    gtk_box_remove (priv->box, GTK_WIDGET (group));
-  else
+  if (gtk_widget_get_parent (GTK_WIDGET (group)) != GTK_WIDGET (priv->box)) {
     ADW_CRITICAL_CANNOT_REMOVE_CHILD (self, group);
+    return;
+  }
+
+  g_ptr_array_remove (priv->groups, group);
+  gtk_box_remove (priv->box, GTK_WIDGET (group));
+}
+
+/**
+ * adw_preferences_page_insert:
+ * @self: a preferences page
+ * @group: the group to add
+ * @index: the index to insert @group a
+ *
+ * Inserts a preferences group to @self at @index.
+ *
+ * If @index is negative or larger than the number of groups, appends the group,
+ * same as [method@PreferencesPage.add].
+ *
+ * Since: 1.8
+ */
+void
+adw_preferences_page_insert (AdwPreferencesPage  *self,
+                             AdwPreferencesGroup *group,
+                             int                  index)
+{
+  AdwPreferencesPagePrivate *priv;
+  GtkWidget *prev_group;
+
+  g_return_if_fail (ADW_IS_PREFERENCES_PAGE (self));
+  g_return_if_fail (ADW_IS_PREFERENCES_GROUP (group));
+
+  priv = adw_preferences_page_get_instance_private (self);
+
+  if (index < 0 || index >= priv->groups->len) {
+    adw_preferences_page_add (self, group);
+    return;
+  }
+
+  g_ptr_array_insert (priv->groups, index, group);
+
+  if (index == 0) {
+    gtk_box_prepend (priv->box, GTK_WIDGET (group));
+    return;
+  }
+
+  prev_group = g_ptr_array_index (priv->groups, index - 1);
+  gtk_box_insert_child_after (priv->box, GTK_WIDGET (group), prev_group);
+}
+
+/**
+ * adw_preferences_page_get_group:
+ * @self: a preferences page
+ * @index: a group index
+ *
+ * Gets the group at @index.
+ *
+ * Can return `NULL` if @index is larger than the number of groups in the page.
+ *
+ * Returns: (transfer none) (nullable): the group at @index
+ *
+ * Since: 1.8
+ */
+AdwPreferencesGroup *
+adw_preferences_page_get_group (AdwPreferencesPage *self,
+                                guint               index)
+{
+  AdwPreferencesPagePrivate *priv;
+
+  g_return_val_if_fail (ADW_IS_PREFERENCES_PAGE (self), NULL);
+
+  priv = adw_preferences_page_get_instance_private (self);
+
+  if (index >= priv->groups->len)
+    return NULL;
+
+  return g_ptr_array_index (priv->groups, index);
 }
 
 /**
