@@ -241,6 +241,19 @@ update_selected (AdwTab *self)
   update_indicator (self);
 }
 
+static void
+update_has_popup (AdwTab *self)
+{
+  if (adw_tab_view_get_menu_model (self->view)) {
+    gtk_accessible_update_property (GTK_ACCESSIBLE (self),
+                                    GTK_ACCESSIBLE_PROPERTY_HAS_POPUP, TRUE,
+                                    -1);
+  } else {
+    gtk_accessible_reset_property (GTK_ACCESSIBLE (self),
+                                   GTK_ACCESSIBLE_PROPERTY_HAS_POPUP);
+  }
+}
+
 static void
 close_idle_cb (AdwTab *self)
 {
@@ -642,6 +655,12 @@ adw_tab_constructed (GObject *object)
   g_signal_connect_object (self->view, "notify::default-icon",
                            G_CALLBACK (update_icons), self,
                            G_CONNECT_SWAPPED);
+
+  g_signal_connect_object (self->view, "notify::menu-model",
+                           G_CALLBACK (update_has_popup), self,
+                           G_CONNECT_SWAPPED);
+
+  update_has_popup (self);
 }
 
 static void
@@ -840,7 +859,7 @@ adw_tab_init (AdwTab *self)
                              CLOSE_BTN_ANIMATION_DURATION, target);
 
   adw_timed_animation_set_easing (ADW_TIMED_ANIMATION (self->close_btn_animation),
-                                  ADW_EASE_IN_OUT_CUBIC);
+                                  ADW_EASE_IN_OUT);
 
   target = adw_callback_animation_target_new ((AdwAnimationTargetFunc)
                                               attention_indicator_animation_value_cb,
@@ -850,7 +869,7 @@ adw_tab_init (AdwTab *self)
                              ATTENTION_INDICATOR_ANIMATION_DURATION, target);
 
   adw_timed_animation_set_easing (ADW_TIMED_ANIMATION (self->needs_attention_animation),
-                                  ADW_EASE_IN_OUT_CUBIC);
+                                  ADW_EASE_IN_OUT);
 }
 
 AdwTab *
@@ -1027,3 +1046,26 @@ adw_tab_set_extra_drag_preload (AdwTab   *self,
 
   gtk_drop_target_set_preload (self->drop_target, preload);
 }
+
+gboolean
+adw_tab_can_click_at (AdwTab *self,
+                      float   x,
+                      float   y)
+{
+  GtkWidget *picked;
+
+  g_return_val_if_fail (ADW_IS_TAB (self), FALSE);
+
+  picked = gtk_widget_pick (GTK_WIDGET (self), x, y, GTK_PICK_DEFAULT);
+
+  if (picked == NULL)
+    return TRUE;
+
+  if (picked == self->close_btn || gtk_widget_is_ancestor (picked, self->close_btn))
+    return FALSE;
+
+  if (picked == self->indicator_btn || gtk_widget_is_ancestor (picked, self->indicator_btn))
+    return FALSE;
+
+  return TRUE;
+}
