@@ -20,6 +20,10 @@
 #include "adw-bottom-sheet-private.h"
 #include "adw-floating-sheet-private.h"
 
+#ifdef GDK_WINDOWING_MACOS
+#include <gdk/macos/gdkmacos.h>
+#endif
+
 typedef struct _CompareInfo CompareInfo;
 
 enum Axis {
@@ -610,6 +614,28 @@ adw_widget_get_ancestor (GtkWidget *widget,
   return widget;
 }
 
+GtkWidget *
+adw_widget_get_nth_child (GtkWidget *widget,
+                          guint      index)
+{
+  GtkWidget *child;
+  guint i;
+
+  g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
+
+  child = gtk_widget_get_first_child (GTK_WIDGET (widget));
+  if (!child)
+    return NULL;
+
+  for (i = 0; i < index; i++) {
+    child = gtk_widget_get_next_sibling (child);
+    if (!child)
+      return NULL;
+  }
+
+  return child;
+}
+
 gboolean
 adw_decoration_layout_prefers_start (const char *layout)
 {
@@ -617,6 +643,11 @@ adw_decoration_layout_prefers_start (const char *layout)
   char **sides;
   int i;
 
+#ifdef GDK_WINDOWING_MACOS
+  if (GDK_IS_MACOS_DISPLAY (gdk_display_get_default ()))
+    return TRUE;
+#endif
+
   if (!layout || !strchr (layout, ':'))
     return FALSE;
 
@@ -779,15 +810,3 @@ adw_tab_on_vertical_keynav_failed (GtkWidget        *widget,
   return gtk_widget_child_focus (toplevel, direction == GTK_DIR_UP ?
                                  GTK_DIR_TAB_BACKWARD : GTK_DIR_TAB_FORWARD);
 }
-
-char *
-adw_get_badge_text (guint badge_number)
-{
-  if (badge_number > 999)
-    return g_strdup ("999+");
-
-  if (!badge_number)
-    return g_strdup ("");
-
-  return g_strdup_printf ("%u", badge_number);
-}
