@@ -56,6 +56,9 @@ test_adw_preferences_page_title (void)
   adw_preferences_page_set_title (page, "Dummy title");
   g_assert_cmpstr (adw_preferences_page_get_title (page), ==, "Dummy title");
 
+  adw_preferences_page_set_title (page, "<b>Dummy title</b>");
+  g_assert_cmpstr (adw_preferences_page_get_title (page), ==, "<b>Dummy title</b>");
+
   adw_preferences_page_set_title (page, NULL);
   g_assert_cmpstr (adw_preferences_page_get_title (page), ==, "");
 
@@ -75,6 +78,9 @@ test_adw_preferences_page_description (void)
   adw_preferences_page_set_description (page, "Dummy description");
   g_assert_cmpstr (adw_preferences_page_get_description (page), ==, "Dummy description");
 
+  adw_preferences_page_set_description (page, "<b>Dummy description</b>");
+  g_assert_cmpstr (adw_preferences_page_get_description (page), ==, "<b>Dummy description</b>");
+
   adw_preferences_page_set_description (page, NULL);
   g_assert_cmpstr (adw_preferences_page_get_description (page), ==, "");
 
